# -*- coding: utf-8 -*-

import typing as T
from datetime import datetime, timezone, timedelta

from acore_server_metadata.api import Server

from .constants import EXPIRE_IN_HOURS


def get_utc_now() -> datetime:
    return datetime.utcnow().replace(tzinfo=timezone.utc)


def get_create_at_expire_at() -> T.Tuple[datetime, datetime]:
    create_at = get_utc_now()
    expire_at = create_at + timedelta(hours=EXPIRE_IN_HOURS)
    return create_at, expire_at


def get_server_status(
    server: Server,
) -> T.Tuple[
    bool,
    bool,
    bool,
    bool,
    T.Optional[str],
    T.Optional[str],
]:
    is_ec2_exists = server.is_ec2_exists()
    is_rds_exists = server.is_rds_exists()
    is_ec2_running = server.is_ec2_running()
    is_rds_running = server.is_rds_running()
    if is_ec2_running:
        ec2_status = server.ec2_inst.status
    else:
        ec2_status = None
    if is_rds_running:
        rds_status = server.rds_inst.status
    else:
        rds_status = None
    return (
        is_ec2_exists,
        is_rds_exists,
        is_ec2_running,
        is_rds_running,
        ec2_status,
        rds_status,
    )



# -*- coding: utf-8 -*-

"""
server monitoring measurement command line user interface.

See :class:`Command` for details.
"""

import fire

from .._version import __version__

from .impl import measure_worldserver


class Command:
    """
    Acore Soap Agent command line interface. All these commands can only be
    used on EC2.
    """

    def hello(self):
        """
        Print welcome message.
        """
        print(f"Hello acore_server_monitoring_measurement {__version__} user!")

    def version(self):
        """
        Print version number.
        """
        print(__version__)

    def measure_worldserver(
        self,
    ):
        """
        Measure WorldServer status.
        """
        measure_worldserver()


def run():
    fire.Fire(Command)



# -*- coding: utf-8 -*-

from acore_server_monitoring_measurement import api


def test():
    _ = api
    _ = api.Ec2RdsStatusMeasurement
    _ = api.WorldServerStatusMeasurement


if __name__ == "__main__":
    from acore_server_monitoring_measurement.tests import run_cov_test

    run_cov_test(__file__, "acore_server_monitoring_measurement.api", preview=False)



# -*- coding: utf-8 -*-

"""
todo: docstring
"""

import psutil
from acore_server_metadata.api import Server
from acore_soap.api import gm
import acore_server_monitoring_core.api as acore_server_monitoring_core

from .utils import get_create_at_expire_at, get_server_status


class WorldServerStatusMeasurement(
    acore_server_monitoring_core.WorldServerStatusMeasurement,
):
    """
    todo: docstring
    """

    @classmethod
    def measure_on_worldserver_ec2(cls):
        """
        todo: docstring
        """
        create_at, expire_at = get_create_at_expire_at()
        server = Server.from_ec2_inside()
        (
            is_ec2_exists,
            is_rds_exists,
            is_ec2_running,
            is_rds_running,
            ec2_status,
            rds_status,
        ) = get_server_status(server)

        try:
            soap_response = gm.ServerInfoRequest().send()
            server_info_response = gm.ServerInfoResponse.from_soap_response(
                soap_response
            )
            connected_players = server_info_response.connected_players
            characters_in_world = server_info_response.characters_in_world
            server_uptime = server_info_response.server_uptime
        except Exception:
            connected_players = None
            characters_in_world = None
            server_uptime = None

        cpu_usage = psutil.cpu_percent(interval=1)
        virtual_memory = psutil.virtual_memory()
        memory_usage = virtual_memory.percent
        total_memory = int(virtual_memory.total / 1000000)
        available_memory = int(virtual_memory.available / 1000000)

        measurement = cls(
            series_id=f"{server.id}-{acore_server_monitoring_core.UseCaseEnum.worldserver_status.value}",
            create_at=create_at,
            expire_at=expire_at,
            is_ec2_exists=is_ec2_exists,
            is_rds_exists=is_rds_exists,
            is_ec2_running=is_ec2_running,
            is_rds_running=is_rds_running,
            ec2_status=ec2_status,
            rds_status=rds_status,
            connected_players=connected_players,
            characters_in_world=characters_in_world,
            server_uptime=server_uptime,
            cpu_usage=cpu_usage,
            memory_usage=memory_usage,
            total_memory=total_memory,
            available_memory=available_memory,
        )
        measurement.save()



# -*- coding: utf-8 -*-

"""
todo: docstring
"""

import typing as T
import os

import boto3
from simple_aws_ec2.api import EC2MetadataCache
from acore_server_metadata.api import Server
import acore_server_monitoring_core.api as acore_server_monitoring_core

from .utils import get_create_at_expire_at, get_server_status


class Ec2RdsStatusMeasurement(
    acore_server_monitoring_core.Ec2RdsStatusMeasurement,
):
    """
    todo: docstring
    """

    @classmethod
    def measure_on_outside(
        cls,
        server_id_list: T.List[str],
        boto_ses: boto3.session.Session,
    ):
        """
        Measure EC2 and RDS status outside the worldserver EC2 instance.

        This method can be reused in many runtime environments,
        such as AWS Lambda, EC2, GitHub Action CI, etc.
        """
        create_at, expire_at = get_create_at_expire_at()
        server = Server.batch_get_server(
            ids=server_id_list,
            ec2_client=boto_ses.client("ec2"),
            rds_client=boto_ses.client("rds"),
        )
        with cls.batch_write() as batch:
            for server_id, server in server.items():
                if server is None:
                    is_ec2_exists = None
                    is_rds_exists = None
                    is_ec2_running = None
                    is_rds_running = None
                    ec2_status = None
                    rds_status = None
                else:
                    (
                        is_ec2_exists,
                        is_rds_exists,
                        is_ec2_running,
                        is_rds_running,
                        ec2_status,
                        rds_status,
                    ) = get_server_status(server)

                measurement = Ec2RdsStatusMeasurement(
                    series_id=f"{server_id}-{acore_server_monitoring_core.UseCaseEnum.ec2_rds_status.value}",
                    create_at=create_at,
                    expire_at=expire_at,
                    is_ec2_exists=is_ec2_exists,
                    is_rds_exists=is_rds_exists,
                    is_ec2_running=is_ec2_running,
                    is_rds_running=is_rds_running,
                    ec2_status=ec2_status,
                    rds_status=rds_status,
                )
                batch.save(measurement)

    @classmethod
    def measure_on_lambda(
        cls,
        server_id_list: T.List[str],
    ):
        """
        Measure EC2 and RDS status on AWS Lambda.
        """
        boto_ses = boto3.session.Session(region_name=os.environ["AWS_DEFAULT_REGION"])
        cls.measure_on_outside(
            server_id_list=server_id_list,
            boto_ses=boto_ses,
        )

    @classmethod
    def measure_on_ec2(
        cls,
        server_id_list: T.List[str],
    ):
        """
        Measure EC2 and RDS status on another EC2 instance.
        """
        boto_ses = EC2MetadataCache.load().get_boto_ses_from_ec2_inside()
        cls.measure_on_outside(
            server_id_list=server_id_list,
            boto_ses=boto_ses,
        )

    @classmethod
    def measure_on_github_action(
        cls,
        server_id_list: T.List[str],
    ):
        """
        Measure EC2 and RDS status on GitHub Action CI.
        """
        raise NotImplementedError



# -*- coding: utf-8 -*-

"""
CLI command business logic implementation.
"""

import pynamodb_mate.api as pm
from pathlib import Path
from acore_server_metadata.api import Server

from ..localmetry import (
    WorldServerStatusMeasurement as Base,
)
from ..paths import path_env_name_cache


if Path("/home/ubuntu").exists():
    try:
        env_name = path_env_name_cache.read_text()
    except FileNotFoundError:  # pragma: no cover
        server = Server.from_ec2_inside()
        env_name = server.env_name
        path_env_name_cache.write_text(env_name)
else:
    env_name = "sbx"


class WorldServerStatusMeasurement(Base):
    class Meta:
        table_name = f"wserver_infra-{env_name}-server_monitoring"
        region = "us-east-1"
        billing_mode = pm.constants.PAY_PER_REQUEST_BILLING_MODE


def measure_worldserver():
    """
    Measure worldserver status once.
    """
    WorldServerStatusMeasurement.measure_on_worldserver_ec2()
